/*
 * lcd-twi.h
 *
 *  Created on: 03.08.2009
 *      Author: tst
 */
/**\file lcd-twi.h
 * \brief LCD Treiber über I2C-Bus
 *
 * Datei zum Einbinden der für die Ansteuerung eines HD44780-kompatiblen 2 Zeilen LCDs über einen an den i2c-Bus angeschloßenen PF8574-Chip benötigten Funktionen.
 *
 */
#ifndef LCDTWI_H_
#define LCDTWI_H_
#include "twi.h"
#include <util/delay.h>
/**
 * \brief Initialisieren des Displays
 *
 * Initialisiert das Display so, dass der Cursor sichbar ist. Achtung, TWI muss vorher innitialisiert werden (siehe twi.h).
 * \param twiaddr Die 7-bit i2c-Adresse des PCF8574-Chips, an dem das LCD angeschlossen ist (msb=0).
 */
int LCDInit(uint8_t twiaddr);

/**
 * \brief Ausgabe eines Zeichens über das LCD
 *
 * Der Buchstabe c wird an der aktuellen Stelle des Cursors ausgegeben. Der Cursor rückt um eine Stelle weiter.
 * \param c Der auszugebende Buchstabe
 */
int LCD_putc(char c);

/**
 * \brief Ausgabe eines Strings auf dem LCD
 *
 * Gibt einen String an der aktuellen Stelle des Cursors aus. Der Anwender ist selbst dafür verantwortlich, dass der komplette String angezeigt werden kann.
 * \param string Der auszugebende String
 *
 */
int LCD_puts(char * string);

/**
 * \brief Setzen der Cursorposition
 *
 * Setzt die Cursorposition im verfügbaren Adressbereich des LCDs.
 * \param position Die neue Position des Cursors. Die defines LINE1 und LINE2 können benutzt werden um zum Anfang der ersten bzw. zweiten Zeile zu gelangen. Durch das hinzuaddieren einer Zahl zwischen 0 und 16 können die Stellen 0 bis 16 der Zeilen erreicht werden.
 */
int LCD_setpos(uint8_t position);

/**
 * \brief Löschen des gesamten LCDs.
 *
 * Löscht das LCD und setzt den Cursor auf die erste Stelle der ersten Zeile.
 */
int LCD_clear();

/**
 * \brief Aktivieren des blinkenden Cursors
 *
 * Zeigt an der Stelle des Cursors ein blinkendes Feld an.
 */
int LCD_enable_blink();
/**
 * \brief Deaktivieren des blinkenden Cursors
 *
 * Zeigt nicht an der Stelle des Cursors ein blinkendes Feld an.
 */
int LCD_disable_blink();

/**
 * \brief Aktivieren des Cursors
 *
 * Aktiviert den Cursor an der nächsten zu beschreibenden Stelle.
 */
int LCD_enable_cursor();

/**
 * \brief Deaktivieren des Cursors
 *
 * Deaktiviert die Anzeige des Cursors und des blinkenden Cursors.
 */
int LCD_disable_cursor();

/**
 * \brief Zurücksetzen des Cursors
 *
 * Setzt den Cursor auf die erste Stelle der ersten Zeile zurück OHNE das löschen des Inhalts. Enspricht der Benutzung von LCD_setpos(LINE1);.
 */
int LCD_return();

// Screen characeteristics
#define CHARSPERLINE 8 //unusual display
#define LCD_LINES 2
#define LINE1 0x00
#define LINE2 0x40
#define LINE3 0x20 //not available!!!
#define LINE4 0x60 //not available!!!

// Display Configuration
#define SETFUNCTION_CMD 0x20
#define FOURBITIF_BIT 0x00
#define EIGHTBITIF_BIT 0x10
#define TWOLINEDISP_BIT 0x08
#define SINGLELINEDISP_BIT 0x00
#define DOT75_BIT 0x00
#define DOT105_BIT 0x04

// Display On/Off Command
#define DISPLAY_ONOFF_CMD 0x08
#define DISPLAY_ON_BIT 0x04
#define CURSOR_ON_BIT 0x02
#define CURSOR_BLINK_ON_BIT 0x01

// Display Clear Command
#define DISPLAY_CLEAR_CMD 0x01

// Return Home Command
#define RETURN_HOME_CMD 0x02

// Entry Mode Command
#define ENTRY_MODE_CMD 0x04
#define INC_CURSOR_BIT 0x02
#define SHIFT_DISPLAY_BIT 0x01

// Shift Command
#define SHIFT_CMD 0x10
#define SHIFT_DISP_BIT 0x08
#define SHIFT_RIGHT_BIT 0x04

// Set DDR-Address
#define DDR_ADDR_CMD 0x80


//Standard display configuration
#define LINES TWOLINEDISP_BIT
#define DOTS DOT75_BIT


#endif /* LCDTWI_H_ */
