/**\file twi.h
 * \brief Two-Wire-Interface (I2C) Funktionen
 *
 * Header-datei für den Zugriff auf das TWI-Interface des ATmega16
 *
 */



/*
 * twi.h
 *
 *  Created on: 02.06.2009
 *      Author: tst
 */

#ifndef TWI_H_
#define TWI_H_

#include <avr/io.h>
#include <util/twi.h>


/**
 * \brief Initialisierung des TWI
 *
 * \param BR der gewünschte Wert des Registers TBRR. Dieser kann dem Datenblatt entnommen werden. Dabei ist jedoch zu beachten, dass die PrescalerBits des TWST Registers auf null gesetzt werden.
 *
 */
int8_t TWI_Init(uint8_t BR);

/**
 * \brief Initialisierung des TWI mit Standardwerten
 *
 * Der TWI wird mit TWBR=72 initialisiert. Dies entspricht einer Frequenz von 100kHz auf SCL.
 *
 */
int8_t TWI_Init_default();



/**
 * \brief Mit dem I2C Bus Daten als Master versenden
 *
 * \param SLA	7-bit Adresse des anzusprechden Geräts. Das MostSignificantBit bleibt unbeachtet und sollte null sein.
 * \param *data	Adresse des Speicherbereichs in dem die zu sendenden Daten gespeichert sind
 * \param count Anzahl der Bytes, die aus dem Speicherbereich data versendet werden sollen
 * \return  Falls der Empfänger eines der Bytes nicht mit ACK bestätigt wurde, wird die Anzahl der übertragenen Bytes zurückgegeben. Ansonsten werden die folgenden Fehlercodes zurückgegeben.
 * \retval  0 Die Übertragung hat, wie gewünscht funktioniert.
 * \retval -1 Undefinierter Fehler
 * \retval -2 Es konnte keine START-Bedingung erzeugt werden
 * \retval -3 Das addressierte Geräte antwortet nicht(ist es angeschlossen?)
 */
int8_t TWIMasterTransmit(int8_t SLA, uint8_t * data, uint8_t count);

/**
 * \brief Mit dem I2C Bus Daten als Master empfangen
 *
 *
 * \param SLA_R	7-bit Addresse des anzusprechden Geräts. Das MostSignificantBit bleibt unbeachtet und sollte null sein.
 * \param *data	Addresse des Speicherbereichs in dem die zu empfangenden Daten gespeichert sind
 * \param length Anzahl der Bytes, die empfangen werden sollen
 * \return  Falls die Übertragung selbst gestört wurde, wird die Anzahl der empfangenen Bytes zurückgegeben. Ansonsten werden die folgenden Fehlercodes zurückgegeben.
 * \retval  0 Die Übertragung hat, wie gewünscht funktioniert.
 * \retval -1 Undefinierter Fehler
 * \retval -2 Es konnte keine START-Bedingung erzeugt werden
 * \retval -3 Das addressierte Geräte antwortet nicht(ist es angeschlossen?)
 */
int8_t TWIMasterReceive(uint8_t SLA_R, uint8_t * data, uint8_t length);

#endif /* TWI_H_ */
